@@ -16,12 +16,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ErrorViewerConsoleListener.cpp,v 1.9 2004/11/15 15:07:37 chpe Exp $
+ *  $Id: ErrorViewerConsoleListener.cpp,v 1.10 2004/12/08 15:33:52 chpe Exp $
  */
 
-#ifdef HAVE_CONFIG_H
+#include "mozilla-config.h"
+
 #include "config.h"
-#endif
 
 #include "error-viewer.h"
 
