@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: sgml-validator.c,v 1.13.2.3 2004/09/26 16:22:39 chpe Exp $
+ *  $Id: sgml-validator.c,v 1.17 2004/10/11 17:27:53 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -271,8 +271,8 @@ convert_to_utf8 (const char *file,
 	char *encoding;
 	char *base;
 	char *buf;
-	int buf_size = 4096;
-	int len;
+	gsize buf_size = 4096;
+	gsize len;
 	GIOChannel *in;
 	GIOChannel *out;
 	GIOStatus status;
@@ -458,7 +458,7 @@ sgml_validator_validate (SgmlValidator *validator,
 	g_return_if_fail (tmp != NULL);
 
 	persist = EPHY_EMBED_PERSIST
-		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
+		(ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 
 	ephy_embed_persist_set_embed (persist, embed);
 	ephy_embed_persist_set_flags (persist, EMBED_PERSIST_NO_VIEW |
